package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityLinkedException;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.CompanyDao;
import dev.fyloz.trial.colorrecipesexplorer.dao.RecipeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CompanyService extends GenericService<Company, CompanyDao> {

    private RecipeDao recipeDao;    // Utilise le Dao pour éviter une erreur au démarrage, citant une récursion (CompanyService -> RecipeService -> CompanyService -> ...)

    @Autowired
    public CompanyService(CompanyDao companyDao, RecipeDao recipeDao) {
        super(companyDao, Company.class);
        this.recipeDao = recipeDao;
    }

    @Override
    public void delete(Company entity) {
        if (isLinkedToRecipes(entity)) throw new EntityLinkedException(type);

        super.delete(entity);
    }

    /**
     * Vérifie si une bannière correspondant à un nom existe.
     *
     * @param name Le nom de la bannière
     * @return Si une bannière correspondant au nome existe
     */
    public boolean existsByName(String name) {
        return dao.existsByName(name);
    }

    /**
     * Vérifie si une bannière est liée à une recette.
     *
     * @param company La bannière
     * @return Si la bannière est liée à une recette
     */
    public boolean isLinkedToRecipes(Company company) {
        return !recipeDao.findAllByCompany(company).isEmpty();
    }

    @Deprecated(since = "1.3.0", forRemoval = true)
    public void deleteIfNotLinked(Company company) {
        if (!isLinkedToRecipes(company)) delete(company);
    }
}
