package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.model.RecipeStep;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.StepDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.List;

@Service
public class StepService extends GenericService<RecipeStep, StepDao> {

    @Autowired
    public StepService(StepDao stepDao) {
        super(stepDao, RecipeStep.class);
    }

    /**
     * Crée une étape pour une recette.
     *
     * @param recipe  La recette
     * @param message Le message de l'étape à créer
     */
    public void createForRecipe(Recipe recipe, String message) {
        save(new RecipeStep(recipe, message));
    }

    /**
     * Crée toutes les étapes pour une recette.
     *
     * @param recipe   La recette
     * @param messages Tous les messages des étapes à créer
     */
    @Transactional
    public void createAllForRecipe(Recipe recipe, List<String> messages) {
        messages.forEach(m -> createForRecipe(recipe, m));
    }

}
