package dev.fyloz.trial.colorrecipesexplorer.web.controller.removers;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityLinkedException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_COMPANY;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_COMPANY_SPECIFIC;

@Controller
public class CompanyRemoverController {

    private CompanyService companyService;

    @Autowired
    public CompanyRemoverController(CompanyService companyService) {
        this.companyService = companyService;
    }

    @GetMapping(REMOVER_COMPANY)
    public ModelAndView getPage(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(REMOVER_COMPANY)
                .addResponseData(ResponseDataType.COMPANIES, companyService.getAll())
                .build();
    }

    @PostMapping(REMOVER_COMPANY_SPECIFIC)
    public ModelAndView removeCompany(@PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder("");

        try {
            companyService.deleteById(id);

            modelResponseBuilder.addResponseCode(ResponseCode.SUCCESS_DELETING_COMPANY, companyService.getById(id));
        } catch (EntityLinkedException ex) {
            modelResponseBuilder.addResponseCode(ResponseCode.COMPANY_LINKED, companyService.getById(id));
        } catch (EntityNotFoundException ex) {
            modelResponseBuilder.addResponseCode(ResponseCode.COMPANY_NOT_FOUND, id);
        }

        return getPage(modelResponseBuilder.build());
    }
}
